@@ -4,9 +4,10 @@ import logging
 import pathlib
 
 import filetype
-
 from las import Client
 
+from lascli.util import json_path
+
 
 def _create_ground_truth_dict(ground_truth_fields, ground_truth_path):
     ground_truth = None
@@ -39,12 +40,11 @@ def get_document(las_client: Client, document_id, download_content, output_conte
         return {**document_resp, 'content': document_resp['content'][:10] + '... [TRUNCATED]'}
 
 
-def list_documents(las_client: Client, consent_id, dataset_id, max_results, next_token):
+def list_documents(las_client: Client, consent_id, dataset_id, **optional_args):
     return las_client.list_documents(
         consent_id=consent_id,
         dataset_id=dataset_id,
-        max_results=max_results,
-        next_token=next_token,
+        **optional_args
     )
 
 
@@ -70,6 +70,7 @@ def create_document(
     dataset_id,
     ground_truth_fields,
     ground_truth_path,
+    **optional_args,
 ):
     content = pathlib.Path(document_path).read_bytes()
 
@@ -86,12 +87,13 @@ def create_document(
         consent_id=consent_id,
         dataset_id=dataset_id,
         ground_truth=ground_truth,
+        **optional_args,
     )
 
 
-def update_document(las_client: Client, document_id, dataset_id, ground_truth_fields, ground_truth_path):
+def update_document(las_client, document_id, dataset_id, ground_truth_fields, ground_truth_path, **optional_args):
     ground_truth = _create_ground_truth_dict(ground_truth_fields, ground_truth_path)
-    return las_client.update_document(document_id, dataset_id=dataset_id, ground_truth=ground_truth)
+    return las_client.update_document(document_id, dataset_id=dataset_id, ground_truth=ground_truth, **optional_args)
 
 
 def create_documents_parser(subparsers):
@@ -116,6 +118,11 @@ def create_documents_parser(subparsers):
     create_document_parser.add_argument('--content-type')
     create_document_parser.add_argument('--consent-id')
     create_document_parser.add_argument('--dataset-id')
+    create_document_parser.add_argument(
+        '--metadata',
+        type=json_path,
+        help='path to json file with whatever you need, maximum limit 4kB',
+    )
     create_document_ground_truth_group = create_document_parser.add_mutually_exclusive_group(required=False)
     create_document_ground_truth_group.add_argument('--ground-truth-fields', metavar='KEY=VALUE', nargs='+')
     create_document_ground_truth_group.add_argument('--ground-truth-path', type=str, help='Path to JSON file')
@@ -127,6 +134,11 @@ def create_documents_parser(subparsers):
     update_document_ground_truth_group = update_document_parser.add_mutually_exclusive_group(required=False)
     update_document_ground_truth_group.add_argument('--ground-truth-fields', metavar='KEY=VALUE', nargs='+')
     update_document_ground_truth_group.add_argument('--ground-truth-path', type=str, help='Path to JSON file')
+    update_document_parser.add_argument(
+        '--metadata',
+        type=json_path,
+        help='path to json file with whatever you need, maximum limit 4kB',
+    )
     update_document_parser.set_defaults(cmd=update_document)
 
     delete_document_parser = subparsers.add_parser('delete')
